-- Configuration --------------------------------------
AUTOTRACKER_ENABLE_DEBUG_LOGGING = false
-------------------------------------------------------
function autotracker_started()
    print("Autotracking Active")
    -- Invoked when the auto-tracker is activated/connected
end

U8_READ_CACHE = 0
U8_READ_CACHE_ADDRESS = 0

U16_READ_CACHE = 0
U16_READ_CACHE_ADDRESS = 0

OFFSET_ITEM_REGISTRY = 0x80356B00 -- Icebound777 was here!

gameActive = false

function InvalidateReadCaches()
    U8_READ_CACHE_ADDRESS = 0
    U16_READ_CACHE_ADDRESS = 0
end

function ReadU8(segment, address)
    if U8_READ_CACHE_ADDRESS ~= address then
        U8_READ_CACHE = segment:ReadUInt8(address)
        U8_READ_CACHE_ADDRESS = address        
    end

    return U8_READ_CACHE
end

function ReadU16(segment, address)
    if U16_READ_CACHE_ADDRESS ~= address then
        U16_READ_CACHE = segment:ReadUInt16(address)
        U16_READ_CACHE_ADDRESS = address        
    end

    return U16_READ_CACHE
end

function updateProgressiveItemFromBytes(segment, code, address, quantity)
    local item = Tracker:FindObjectForCode(code)
    if item then
        local value = 0
        for i = 0, quantity-1, 1 do
            if ReadU8(segment, address+i) > 0 then
                value = value + 1
            end
        end
        if item.CurrentStage ~= value then
            print(item.Name .. ": " .. value)
            item.CurrentStage = value
        end
    end
end

function updateLetters(segment, address)
    local item = Tracker:FindObjectForCode("letters")
    if item then
        local value = 0
        for i = 0, 30, 1 do
            if ReadU8(segment, address+i) > 0 and i ~= 9 and i~= 12 and i~= 13 and i~= 14 and i ~= 23 and i~= 29 then
                value = value + 1
            end
        end
        if value > 3 then
            value = 3
        end
        if item.CurrentStage ~= value then
            print(item.Name .. ": " .. value)
            item.CurrentStage = value
        end
    end
end

function updateProgressiveToggleStateFromByte(segment, code, address)
    local item = Tracker:FindObjectForCode(code)
    if item then
        local value = ReadU8(segment, address)
        if value > 0 then
            if item.ItemState.toggle == false then
                print(item.Name .. " left object Obtained")
                item.ItemState.toggle = true
            end
        else
            item.ItemState.toggle = false
        end
        item.ItemState:updateIcon()
    end
end

function updateProgressiveToggleStateFromByteAndFlag(segment, code, address, flag)
    local item = Tracker:FindObjectForCode(code)
    if item then
        local value = ReadU8(segment, address)
        local flagTest = value & flag
        if flagTest ~= 0 then
            if item.ItemState.toggle == false then
                print(item.Name .. " left object Obtained")
                item.ItemState.toggle = true
            end
        else
            item.ItemState.toggle = false
        end
        item.ItemState:updateIcon()
    end
end

function updateProgressiveToggleStageFromBytes(segment, code, address, quantity)
    local item = Tracker:FindObjectForCode(code)
    if item then
        local value = 0
        for i = 0, quantity-1, 1 do
            if ReadU8(segment, address+i) > 0 then
                value = value + 1
            end
        end
        if item.ItemState.stage ~= value+1 then
            print(item.Name .. ": " .. value)
            item.ItemState.stage = value+1
            item.ItemState:updateIcon()
        end
    end
end

function updateProgressiveToggleStageFromByte(segment, code, address)
    local item = Tracker:FindObjectForCode(code)
    if item then
        local value = ReadU8(segment, address)
        if item.ItemState.stage ~= value+1 then
            print(item.Name .. ": " .. value)
            item.ItemState.stage = value+1
            item.ItemState:updateIcon()
        end
    end
end

function updateProgressiveItemFromByte(segment, code, address)
    local item = Tracker:FindObjectForCode(code)
    if item then
        local value = ReadU8(segment, address)
        if item.ItemState.stage ~= value then
            print(item.Name .. ": " .. value)
            item.CurrentStage = value
        end
    end
end

function updateToggleItemFromByte(segment, code, address)
    local item = Tracker:FindObjectForCode(code)
    if item then
        local value = ReadU8(segment, address)
        if value > 0 then
            if item.Active == false then
                print(item.Name .. " obtained")
                item.Active = true
            end
        else
            item.Active = false
        end
    end
end

function updateToggleItemFromByteAndFlag(segment, code, address, flag)
    local item = Tracker:FindObjectForCode(code)
    if item then
        local value = ReadU8(segment, address)
        if value & flag ~= 0 then
            if item.Active == false then
                print(item.Name .. " obtained")
                item.Active = true
            end
        else
            item.Active = false
        end
    end
end
function updateGameStatus()
    -- check value of LuckyStar item (always there when game is running)
    if AutoTracker:ReadU8(OFFSET_ITEM_REGISTRY+0x7) > 0 then
        gameActive = true
    else
        gameActive = false
    end
end

function ScanKeyItems(segment) --for debugging
    for i = 0, 0x184, 1 do
        if ReadU8(segment, OFFSET_ITEM_REGISTRY+i) > 0 then
            print ("Address " .. string.format("%x",i) .. ": " .. ReadU8(segment, OFFSET_ITEM_REGISTRY+i))
        end
    end
end

function updateKeyItems(segment)

    InvalidateReadCaches()

    if AUTOTRACKER_ENABLE_ITEM_TRACKING then
        updateGameStatus()
        if gameActive then
            updateToggleItemFromByte(segment,"ultrastone",0xF+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"pulsestone",0x12+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"palacekey",0x14+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"lunarstone",0x15+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"pyramidstone",0x16+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"diamondstone",0x17+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"shell",0x19+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"weight",0x1C+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"boosportrait",0x1D+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"crystalberry",0x1E+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"storeroom",0x20+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"toytrain",0x21+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"record",0x22+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"fryingpan",0x23+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"dictionary",0x24+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"cookbook",0x29+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"jaderaven",0x2A+OFFSET_ITEM_REGISTRY)
            updateProgressiveToggleStageFromBytes(segment, "KleSeeds", 0x2B+OFFSET_ITEM_REGISTRY, 4)
            updateProgressiveItemFromBytes(segment, "seeds", 0x2B+OFFSET_ITEM_REGISTRY, 4)
            updateToggleItemFromByte(segment,"calculator",0x30+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"bucket",0x31+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"scarf",0x32+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"redkey",0x33+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"bluekey",0x34+OFFSET_ITEM_REGISTRY)
            updateLetters(segment, 0x36+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"artifact",0x53+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"dolly",0x56+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"waterstone",0x57+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"magicalbean",0x58+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"fertilesoil",0x59+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"miraclewater",0x5A+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"volcanovase",0x5B+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"lyrics",0x67+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"melody",0x68+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"mailbag",0x69+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"oddkey",0x6B+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"starstone",0x6C+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"warehouse",0x79+OFFSET_ITEM_REGISTRY) 
            updateProgressiveToggleStageFromBytes(segment, "EldKeys", 0x16D+OFFSET_ITEM_REGISTRY, 4)
            updateProgressiveItemFromBytes(segment, "fortkey", 0x16D+OFFSET_ITEM_REGISTRY, 4)
            updateProgressiveToggleStageFromBytes(segment, "MamKeys", 0x171+OFFSET_ITEM_REGISTRY, 4)
            updateProgressiveItemFromBytes(segment, "ruinskey", 0x171+OFFSET_ITEM_REGISTRY, 4)
            updateProgressiveToggleStageFromBytes(segment, "SkoKeys", 0x175+OFFSET_ITEM_REGISTRY, 3)
            updateProgressiveItemFromBytes(segment, "tubbakey", 0x175+OFFSET_ITEM_REGISTRY, 3)
            updateProgressiveItemFromBytes(segment, "bowserkey", 0x178+OFFSET_ITEM_REGISTRY, 5)
            updateProgressiveItemFromBytes(segment, "prisonkey", 0x17D+OFFSET_ITEM_REGISTRY, 2)
            updateProgressiveToggleStageFromBytes(segment, "red2blueberries", 0x180+OFFSET_ITEM_REGISTRY, 2)
            updateProgressiveToggleStateFromByte(segment, "red2blueberries", 0x182+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"yellowberry",0x183+OFFSET_ITEM_REGISTRY)
            updateToggleItemFromByte(segment,"purpleberry",0x184+OFFSET_ITEM_REGISTRY) 
        end
    end
end

function updatePartners(segment)

    InvalidateReadCaches()

    if AUTOTRACKER_ENABLE_ITEM_TRACKING then
        updateGameStatus()
        if gameActive then
            updateProgressiveToggleStateFromByte(segment, "goombario", 0x8010f2ae)
            updateProgressiveToggleStageFromByte(segment, "goombario", 0x8010f2ad)
            updateProgressiveToggleStateFromByte(segment, "kooper", 0x8010f2b6)
            updateProgressiveToggleStageFromByte(segment, "kooper", 0x8010f2b5)
            updateProgressiveToggleStateFromByte(segment, "bombette", 0x8010f2be)
            updateProgressiveToggleStageFromByte(segment, "bombette", 0x8010f2bd)
            updateProgressiveToggleStateFromByte(segment, "parakarry", 0x8010f2c6)
            updateProgressiveToggleStageFromByte(segment, "parakarry", 0x8010f2c5)
            updateProgressiveToggleStateFromByte(segment, "bow", 0x8010f2ee)
            updateProgressiveToggleStageFromByte(segment, "bow", 0x8010f2ed)
            updateProgressiveToggleStateFromByte(segment, "sushi", 0x8010f2de)
            updateProgressiveToggleStageFromByte(segment, "sushi", 0x8010f2dd)
            updateProgressiveToggleStateFromByte(segment, "watt", 0x8010f2d6)
            updateProgressiveToggleStageFromByte(segment, "watt", 0x8010f2d5)
            updateProgressiveToggleStateFromByte(segment, "lakilester", 0x8010f2e6)
            updateProgressiveToggleStageFromByte(segment, "lakilester", 0x8010f2e5)
        end
    end
end

function updateEquipment(segment)

    InvalidateReadCaches()

    if AUTOTRACKER_ENABLE_ITEM_TRACKING then
        updateGameStatus()
        if gameActive then
            updateProgressiveItemFromByte(segment,"boots",0x8010f290)
            updateProgressiveItemFromByte(segment,"hammer",0x8010f291)
        end
    end
end

function updateChapters(segment)

    InvalidateReadCaches()

    if AUTOTRACKER_ENABLE_ITEM_TRACKING then
        updateGameStatus()
        if gameActive then
            updateToggleItemFromByteAndFlag(segment, "eldstar", 0x80357017, 0x02)
            updateProgressiveToggleStateFromByteAndFlag(segment, "EldKeys", 0x80357017, 0x02)
            updateToggleItemFromByteAndFlag(segment, "mamar", 0x80357017, 0x04)
            updateProgressiveToggleStateFromByteAndFlag(segment, "MamKeys", 0x80357017, 0x04)
            updateToggleItemFromByteAndFlag(segment, "skolar", 0x80357017, 0x08)
            updateProgressiveToggleStateFromByteAndFlag(segment, "SkoKeys", 0x80357017, 0x08)
            updateToggleItemFromByteAndFlag(segment, "muskular", 0x80357017, 0x10)
            updateToggleItemFromByteAndFlag(segment, "misstar", 0x80357017, 0x20)
            updateProgressiveToggleStateFromByteAndFlag(segment, "KleSeeds", 0x80357017, 0x40)
            updateToggleItemFromByteAndFlag(segment, "kelvar", 0x80357017, 0x40)
            updateToggleItemFromByteAndFlag(segment, "kalmar", 0x80357017, 0x80)
        end
    end
end

-- Run the in-game status check more frequently (every 250ms) to catch save/quit scenarios more effectively
ScriptHost:AddMemoryWatch("Key Items", OFFSET_ITEM_REGISTRY, 0x185, updateKeyItems, 250)
ScriptHost:AddMemoryWatch("Partners", 0x8010f2ad, 0x42, updatePartners, 250)
ScriptHost:AddMemoryWatch("Equipment", 0x8010f290, 0x02, updateEquipment, 250)
ScriptHost:AddMemoryWatch("Chapters", 0x80357017, 0x01, updateChapters, 250)
